#ifndef MGLPP_TEXT_HPP
#define MGLPP_TEXT_HPP

#include <string>
#include "Drawable.hpp"
#include "../system/FloatRect.hpp"

extern "C" {
#include <mgl/graphics/text.h>
}

namespace mgl {
    class Font;
    class Text : public Drawable {
    public:
        Text();
        Text(std::string str, Font &font);
        Text(std::string str, vec2f position, Font &font);
        Text(const Text &other);
        Text& operator=(const Text &other);
        Text(Text &&other);
        ~Text();

        void set_position(vec2f position) override;
        vec2f get_position() const override;

        void set_color(Color color) override;
        // If |max_width| is 0 then the text has no max width
        void set_max_width(float max_width);
        // If |max_rows| is 0 then the text can display an unlimited amount of rows
        void set_max_rows(unsigned int max_rows);

        FloatRect get_bounds();
        void set_string(std::string str);
        const std::string& get_string() const;
        void append_string(const std::string &str);

        // Returns the visual position of a character from its index.
        // If the index is out of range, then the position of the end of the string is returned.
        vec2f find_character_pos(size_t index);

        Font* get_font();
    protected:
        void draw(Window &window) override;
    private:
        mgl_text text;
        Font *font;
        std::string str;
    };
}

#endif /* MGLPP_TEXT_HPP */
